/*
 * Copyright (C) 2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef DOMAIN_INFO_REPLY_HH_0E8302018D5E41A1A4A60FD3F4D73AA8
#define DOMAIN_INFO_REPLY_HH_0E8302018D5E41A1A4A60FD3F4D73AA8

#include "src/contact/contact_common_types.hh"
#include "src/domain/domain_common_types.hh"
#include "src/domain/domain_info_request.hh"
#include "src/keyset/keyset_common_types.hh"
#include "src/nsset/nsset_common_types.hh"
#include "src/object/object_common_types.hh"
#include "src/registrar/registrar_common_types.hh"
#include "src/util/printable.hh"
#include "src/exceptions.hh"

#include <boost/optional.hpp>

#include <string>
#include <vector>

namespace Fred {
namespace Registry {
namespace Domain {

struct DomainInfoReply
{
    struct Data : Util::Printable<Data>
    {
        Fqdn fqdn;
        DomainId domain_id;
        DomainHistoryId domain_history_id;
        boost::optional<Nsset::NssetId> nsset;
        boost::optional<Keyset::KeysetId> keyset;
        Contact::ContactId registrant;
        Registrar::RegistrarHandle sponsoring_registrar;
        std::vector<Contact::ContactId> administrative_contacts;
        ExpirationTimestamp expires_at;
        AuthInfo auth_info;
        Object::ObjectEvents representative_events;
        std::string to_string() const;
    };
    struct Exception
    {
        struct DomainDoesNotExist : Registry::Exception
        {
            explicit DomainDoesNotExist(const DomainInfoRequest& src);
        };
    };
};

} // namespace Fred::Registry::Domain
} // namespace Fred::Registry
} // namespace Fred

#endif
