/*
 * Copyright (C) 2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */
#include "src/nsset/nsset_info.hh"

#include "src/fred_unwrap.hh"
#include "src/util/strong_type.hh"

#include "libfred/db_settings.hh"
#include "libfred/registrable_object/nsset/info_nsset.hh"

namespace Fred {
namespace Registry {
namespace Nsset {

NssetInfoReply::Data nsset_info(const NssetInfoRequest& request)
{
    LibFred::OperationContextCreator ctx_provider;
    LibFred::OperationContextLockingForShare ctx(ctx_provider);
    try
    {
        if (request.nsset_history_id != boost::none)
        {
            const auto nsset_history_uuid = LibFred::RegistrableObject::make_history_uuid_of<LibFred::Object_Type::nsset>(
                    get_raw_value_from(*request.nsset_history_id));
            return fred_unwrap(LibFred::InfoNssetByHistoryUuid(nsset_history_uuid).exec(ctx));
        }
        const auto nsset_uuid = LibFred::RegistrableObject::make_uuid_of<LibFred::Object_Type::nsset>(
                get_raw_value_from(request.nsset_id));
        return fred_unwrap(LibFred::InfoNssetByUuid(nsset_uuid).exec(ctx));
    }
    catch (const LibFred::InfoNssetByUuid::Exception& e)
    {
        if (e.is_set_unknown_nsset_uuid())
        {
            throw NssetInfoReply::Exception::NssetDoesNotExist(request);
        }
        throw InternalServerError("unexpected problem signalized by LibFred::NssetInfoByUuid::Exception");;
    }
    catch (const LibFred::InfoNssetByHistoryUuid::Exception& e)
    {
        if (e.is_set_unknown_nsset_history_uuid())
        {
            throw NssetInfoReply::Exception::NssetDoesNotExist(request);
        }
        throw InternalServerError("unexpected problem signalized by LibFred::InfoNssetByHistoryUuid::Exception");;
    }
    catch (const std::exception& e)
    {
        throw InternalServerError(std::string("expected unknown exception: ") + e.what());
    }
    catch (...)
    {
        throw InternalServerError("unexpected unknown exception");
    }
}

} // namespace Fred::Registry::Nsset
} // namespace Fred::Registry
} // namespace Fred
