/*
 * Copyright (C) 2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef CONTACT_HH_A9DA05F476FF258971F5819F007678A0//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define CONTACT_HH_A9DA05F476FF258971F5819F007678A0

#include "libfred/registrable_object/contact/create_contact.hh"
#include "libfred/registrable_object/contact/info_contact.hh"

#include "src/util/strong_type.hh"

#include "test/setup/fixture/registrar.hh"

#include <string>
#include <utility>

namespace Test {

class Contact
{
private:
    template <typename T, typename N>
    using PrintableStrongType = Fred::Registry::Util::StrongType<T, N, Fred::Registry::Util::Skill::Printable>;
public:
    using Handle = PrintableStrongType<std::string, struct Handle_>;
    static Handle make_handle(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<Handle>(raw_value);
    }
    using Name = PrintableStrongType<std::string, struct Name_>;
    static Name make_name(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<Name>(raw_value);
    }
    using Organization = PrintableStrongType<std::string, struct Organization_>;
    static Organization make_organization(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<Organization>(raw_value);
    }
    using Authinfo = PrintableStrongType<std::string, struct Authinfo_>;
    static Authinfo make_authinfo(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<Authinfo>(raw_value);
    }
    using PlaceAddress = PrintableStrongType<LibFred::Contact::PlaceAddress, struct PlaceAddress_>;
    static PlaceAddress make_place_address(const LibFred::Contact::PlaceAddress& raw_value)
    {
        return Fred::Registry::Util::make_strong<PlaceAddress>(raw_value);
    }
    using Telephone = PrintableStrongType<std::string, struct Telephone_>;
    static Telephone make_telephone(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<Telephone>(raw_value);
    }
    using Fax = PrintableStrongType<std::string, struct Fax_>;
    static Fax make_fax(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<Fax>(raw_value);
    }
    using Email = PrintableStrongType<std::string, struct Email_>;
    static Email make_email(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<Email>(raw_value);
    }
    using NotifyEmail = PrintableStrongType<std::string, struct NotifyEmail_>;
    static NotifyEmail make_notify_email(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<NotifyEmail>(raw_value);
    }
    using VatIdentificationNumber = PrintableStrongType<std::string, struct VatIdentificationNumber_>;
    static VatIdentificationNumber make_vat_identification_number(const std::string& raw_value)
    {
        return Fred::Registry::Util::make_strong<VatIdentificationNumber>(raw_value);
    }
    using PersonalIdUnion = PrintableStrongType<LibFred::PersonalIdUnion, struct PersonalIdUnion_>;
    static PersonalIdUnion make_personal_id(const LibFred::PersonalIdUnion& raw_value)
    {
        return Fred::Registry::Util::make_strong<PersonalIdUnion>(raw_value);
    }
    using ContactAddressList = PrintableStrongType<LibFred::ContactAddressList, struct ContactAddressList_>;
    static ContactAddressList make_contact_address_list(const LibFred::ContactAddressList& raw_value)
    {
        return Fred::Registry::Util::make_strong<ContactAddressList>(raw_value);
    }
    struct Disclose
    {
        using Name = PrintableStrongType<bool, struct Name_>;
        static Name name()
        {
            return Fred::Registry::Util::make_strong<Name>(true);
        }
        using Organization = PrintableStrongType<bool, struct Organization_>;
        static Organization organization()
        {
            return Fred::Registry::Util::make_strong<Organization>(true);
        }
        using Address = PrintableStrongType<bool, struct Address_>;
        static Address address()
        {
            return Fred::Registry::Util::make_strong<Address>(true);
        }
        using Telephone = PrintableStrongType<bool, struct Telephone_>;
        static Telephone telephone()
        {
            return Fred::Registry::Util::make_strong<Telephone>(true);
        }
        using Fax = PrintableStrongType<bool, struct Fax_>;
        static Fax fax()
        {
            return Fred::Registry::Util::make_strong<Fax>(true);
        }
        using Email = PrintableStrongType<bool, struct Email_>;
        static Email email()
        {
            return Fred::Registry::Util::make_strong<Email>(true);
        }
        using NotifyEmail = PrintableStrongType<bool, struct NotifyEmail_>;
        static NotifyEmail notify_email()
        {
            return Fred::Registry::Util::make_strong<NotifyEmail>(true);
        }
        using VatIdentificationNumber = PrintableStrongType<bool, struct VatIdentificationNumber_>;
        static VatIdentificationNumber vat_identification_number()
        {
            return Fred::Registry::Util::make_strong<VatIdentificationNumber>(true);
        }
        using Ident = PrintableStrongType<bool, struct Ident_>;
        static Ident ident()
        {
            return Fred::Registry::Util::make_strong<Ident>(true);
        }
    };
    struct Hide
    {
        static Disclose::Name name()
        {
            return Fred::Registry::Util::make_strong<Disclose::Name>(false);
        }
        static Disclose::Organization organization()
        {
            return Fred::Registry::Util::make_strong<Disclose::Organization>(false);
        }
        static Disclose::Address address()
        {
            return Fred::Registry::Util::make_strong<Disclose::Address>(false);
        }
        static Disclose::Telephone telephone()
        {
            return Fred::Registry::Util::make_strong<Disclose::Telephone>(false);
        }
        static Disclose::Fax fax()
        {
            return Fred::Registry::Util::make_strong<Disclose::Fax>(false);
        }
        static Disclose::Email email()
        {
            return Fred::Registry::Util::make_strong<Disclose::Email>(false);
        }
        static Disclose::NotifyEmail notify_email()
        {
            return Fred::Registry::Util::make_strong<Disclose::NotifyEmail>(false);
        }
        static Disclose::VatIdentificationNumber vat_identification_number()
        {
            return Fred::Registry::Util::make_strong<Disclose::VatIdentificationNumber>(false);
        }
        static Disclose::Ident ident()
        {
            return Fred::Registry::Util::make_strong<Disclose::Ident>(false);
        }
    };
    const LibFred::InfoContactData& get_data()const;
    template <typename ...T>
    static Contact init(
            LibFred::OperationContext& ctx,
            const Handle& handle,
            const Registrar::Handle& sponsoring_registrar,
            T&& ...values)
    {
        LibFred::CreateContact create_contact_op(get_raw_value_from(handle), get_raw_value_from(sponsoring_registrar));
        set_bunch_of_attributes(create_contact_op, std::forward<T>(values)...);
        const auto create_contact_result = create_contact_op.exec(ctx);
        return Contact(
                LibFred::InfoContactById(create_contact_result.create_object_result.object_id).exec(ctx).info_contact_data);
    }
private:
    explicit Contact(const LibFred::InfoContactData&);
    template <typename T>
    static void set_attribute(LibFred::CreateContact&, const T&);
    template <typename ...Ts>
    static void set_bunch_of_attributes(const LibFred::CreateContact&, const Ts&...)
    {
        static_assert(sizeof...(Ts) == 0, "must be used with an empty bunch");
    }
    template <typename T, typename ...Ts>
    static void set_bunch_of_attributes(LibFred::CreateContact& create_contact_op, const T& value, const Ts& ...bunch)
    {
        set_attribute(create_contact_op, value);
        set_bunch_of_attributes(create_contact_op, bunch...);
    }
    LibFred::InfoContactData data_;
};

}//namespace Test

#endif//CONTACT_HH_A9DA05F476FF258971F5819F007678A0
